@@ -326,7 +326,7 @@ clamp (const T& a, const T& low, const T& high)
     // should result in just a max and min instruction, thats it.
     // This implementation is courtesy of Alex Wells, Intel, via OSL.
     T val = a;
-    if (!(low < val))  // Forces clamp(NaN,low,high) to return low
+    if (!(low <= val))  // Forces clamp(NaN,low,high) to return low
         val = low;
     if (val > high)
         val = high;
@@ -361,6 +361,25 @@ clamp (const simd::vfloat16& a, const simd::vfloat16& low, const simd::vfloat16&
 {
     return simd::min (high, simd::max (low, a));
 }
+
+// Specialization of clamp for vint4
+template<> OIIO_FORCEINLINE simd::vint4
+clamp (const simd::vint4& a, const simd::vint4& low, const simd::vint4& high)
+{
+    return simd::min (high, simd::max (low, a));
+}
+
+template<> OIIO_FORCEINLINE simd::vint8
+clamp (const simd::vint8& a, const simd::vint8& low, const simd::vint8& high)
+{
+    return simd::min (high, simd::max (low, a));
+}
+
+template<> OIIO_FORCEINLINE simd::vint16
+clamp (const simd::vint16& a, const simd::vint16& low, const simd::vint16& high)
+{
+    return simd::min (high, simd::max (low, a));
+}
 #endif
 
 
@@ -838,7 +857,7 @@ void convert_type (const S *src, D *dst, size_t n, D _min, D _max)
     }
     typedef typename big_enough_float<D>::float_t F;
     F scale = std::numeric_limits<S>::is_integer ?
-        ((F)1.0)/std::numeric_limits<S>::max() : (F)1.0;
+        (F(1)) / F(std::numeric_limits<S>::max()) : F(1);
     if (std::numeric_limits<D>::is_integer) {
         // Converting to an integer-like type.
         F min = (F)_min;  // std::numeric_limits<D>::min();
@@ -1050,7 +1069,7 @@ convert_type (const S &src)
     }
     typedef typename big_enough_float<D>::float_t F;
     F scale = std::numeric_limits<S>::is_integer ?
-        ((F)1.0)/std::numeric_limits<S>::max() : (F)1.0;
+        F(1) / F(std::numeric_limits<S>::max()) : F(1);
     if (std::numeric_limits<D>::is_integer) {
         // Converting to an integer-like type.
         F min = (F) std::numeric_limits<D>::min();
@@ -1518,7 +1537,7 @@ OIIO_FORCEINLINE OIIO_HOSTDEVICE int fast_rint (float x) {
     // used by sin/cos/tan range reduction
 #if OIIO_SIMD_SSE >= 4
     // single roundps instruction on SSE4.1+ (for gcc/clang at least)
-    return static_cast<int>(rintf(x));
+    return static_cast<int>(std::rint(x));
 #else
     // emulate rounding by adding/substracting 0.5
     return static_cast<int>(x + copysignf(0.5f, x));
