@@ -264,7 +264,13 @@ bool
 Strutil::get_rest_arguments(const std::string& str, std::string& base,
                             std::map<std::string, std::string>& result)
 {
-    std::string::size_type mark_pos = str.find_first_of("?");
+    // Disregard the Windows long path question style prefix "\\?\"
+    static const std::string longPathQPrefix("\\\\?\\");
+    auto find_start_pos = starts_with(str, longPathQPrefix)
+                              ? longPathQPrefix.size()
+                              : size_t(0);
+
+    std::string::size_type mark_pos = str.find_first_of("?", find_start_pos);
     if (mark_pos == std::string::npos) {
         base = str;
         return true;
