@@ -180,6 +180,23 @@ PNGOutput::open(const std::string& name, const ImageSpec& userspec,
         png_set_compression_strategy(m_png, Z_DEFAULT_STRATEGY);
     }
 
+    png_set_filter(m_png, 0,
+                   spec().get_int_attribute("png:filter", PNG_NO_FILTERS));
+    // https://www.w3.org/TR/PNG-Encoders.html#E.Filter-selection
+    // https://www.w3.org/TR/PNG-Rationale.html#R.Filtering
+    // The official advice is to PNG_NO_FILTER for palette or < 8 bpp
+    // images, but we and one of the others may be fine for >= 8 bit
+    // greyscale or color images (they aren't very prescriptive, noting that
+    // different flters may be better for different images.
+    // We have found the tradeoff complex, in fact as seen in
+    // https://github.com/OpenImageIO/oiio/issues/2645
+    // where we showed that across several images, 8 (PNG_FILTER_NONE --
+    // don't ask me how that's different from PNG_NO_FILTERS) had the
+    // fastest performance, but also made the largest files. I had trouble
+    // finding a filter choice that for "ordinary" images consistently
+    // performed better than the default on both time and resulting file
+    // size. So for now, we are keeping the default 0 (PNG_NO_FILTERS).
+
     PNG_pvt::write_info(m_png, m_info, m_color_type, m_spec, m_pngtext,
                         m_convert_alpha, m_gamma);
 
